@@ -1,10 +1,6 @@
 package HTTP::Engine::Middleware;
-use 5.00800;
-use Any::Moose;
-use Any::Moose (
-    '::Util' => [qw/apply_all_roles/],
-);
-our $VERSION = '0.11_01';
+use Mouse;
+our $VERSION = '0.01';
 
 use Carp ();
 
@@ -15,17 +11,11 @@ has 'middlewares' => (
 );
 
 has '_instance_of' => (
-    is      => 'rw',
+    is      => 'ro',
     isa     => 'HashRef',
     default => sub { +{} },
 );
 
-has '_instance_ary_ex' => (
-    is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { +[] },
-);
-
 has 'method_class' => (
     is      => 'ro',
     isa     => 'Str',
@@ -38,8 +28,8 @@ has 'diecatch' => (
 
 sub init_class {
     my $klass = shift;
-    my $meta  = any_moose('::Meta::Class')->initialize($klass);
-    $meta->superclasses(any_moose('::Object'))
+    my $meta  = Mouse::Meta::Class->initialize($klass);
+    $meta->superclasses('Mouse::Object')
         unless $meta->superclasses;
 
     no strict 'refs';
@@ -58,11 +48,7 @@ sub import {
 
     init_class($caller);
 
-    if (Any::Moose::is_moose_loaded()) {
-        Moose->import({ into_level => 1 });
-    } else {
-        Mouse->export_to_level( 1 );
-    }
+    Mouse->export_to_level(1);    
 
     no strict 'refs';
     *{"$caller\::__MIDDLEWARE__"} = sub {
@@ -71,128 +57,121 @@ sub import {
         __MIDDLEWARE__($caller);
     };
 
-    *{"$caller\::before_handle"}     = sub (&) { goto \&before_handle     };
-    *{"$caller\::after_handle"}      = sub (&) { goto \&after_handle      };
-    *{"$caller\::middleware_method"} = sub     { goto \&middleware_method };
+    *{"$caller\::before_handle"}     = sub (&) { goto \&before_handle; };
+    *{"$caller\::after_handle"}      = sub (&) { goto \&after_handle; };
+    *{"$caller\::middleware_method"} = sub { goto \&middleware_method; };
+    *{"$caller\::outer_middleware"}  = sub ($) { goto \&outer_middleware; };
+    *{"$caller\::inner_middleware"}  = sub ($)  { goto \&inner_middleware; };
 }
 
 sub __MIDDLEWARE__ {
     my ( $caller, ) = @_;
 
-    Any::Moose::unimport;
-    apply_all_roles( $caller, 'HTTP::Engine::Middleware::Role' );
-
+    Mouse::unimport;
+    Mouse::Util::apply_all_roles( $caller, 'HTTP::Engine::Middleware::Role' );
     $caller->meta->make_immutable( inline_destructor => 1 );
     "MIDDLEWARE";
 }
 
-BEGIN {
-    no strict 'refs';
-    for my $meth (
-        qw(before_handle after_handle middleware_method)
-      )
-    {
-        *{__PACKAGE__ . "::$meth"} = sub {
-            Carp::croak("Can't call ${meth} function outside Middleware's load phase");
-        };
-    }
-};
+sub before_handle {
+    Carp::croak "Can't call before_handle function outside Middleware's load phase";
+}
+
+sub after_handle {
+    Carp::croak "Can't call after_handle function outside Middleware's load phase";
+}
+
+sub middleware_method {
+    Carp::croak "Can't call middleware_method function outside Middleware's load phase";
+}
+
+sub outer_middleware {
+    Carp::croak "Can't call outer_middleware function outside Middleware's load phase";
+}
+
+sub inner_middleware {
+    Carp::croak "Can't call inner_middleware function outside Middleware's load phase";
+}
 
-# this method's return value is indefinite.
 sub install {
     my($self, @middlewares) = @_;
 
-    my $args = $self->_build_args(@middlewares);
-    $self->_create_middleware_instance($args);
-}
+    my %config;
+    for my $middleware (@middlewares) {
+        if (ref($middleware) eq 'HASH') {
+            $config{$self->middlewares->[-1]} = $middleware;
+            next;
+        }
 
-# this module accepts
-#  $mw->install(qw/HTTP::Engine::Middleware::Foo/);
-# and
-#  $mw->install('HTTP::Engine::Middleware::Foo' => { arg1 => 'foo'});
-sub _build_args {
-    my $self = shift;
+        $config{$middleware} = {};
+        push @{ $self->middlewares }, $middleware;
+    }
 
-    # basis of Data::OptList
-    my @middlewares;
-    my $max = scalar(@_);
-    for (my $i = 0; $i < $max ; $i++) {
-        if ($i + 1 < $max && ref($_[$i + 1])) {
-            push @middlewares, [ $_[$i++] => $_[$i] ];
+    # load and create instance
+    my %dependend ;
+    while (my($name, $config) = each %config) {
+        $dependend{$name} = { outer => [], inner => [] };
+
+        unless ($name->can('before_handles')) {
+            # init declear
+            my @before_handles;
+            my @after_handles;
+
+            no strict 'refs';
+            no warnings 'redefine';
+
+            local *before_handle = sub { push @before_handles, @_ };
+            local *after_handle  = sub { push @after_handles, @_ };
+            local *middleware_method = $self->method_class ? sub {
+                my($method, $code) = @_;
+                my $method_class = $self->method_class;
+                if ($method =~ /^(.+)\:\:([^\:]+)$/) {
+                    ($method_class, $method) = ($1, $2);
+                }
+                no strict 'refs';
+                *{"$name\::$method"}         = $code;
+                *{"$method_class\::$method"} = $code;
+            } : sub {};
+            local *outer_middleware = sub { push @{ $dependend{$name}->{outer} }, $_[0] };
+            local *inner_middleware = sub { push @{ $dependend{$name}->{inner} }, $_[0] };
+            local $@;
+            Mouse::load_class($name);
+            $@ and Carp::croak $@;
+
+            *{"$name\::_before_handles"}    = sub () { @before_handles };
+            *{"$name\::_after_handles"}     = sub () { @after_handles };
+            *{"$name\::_outer_middlewares"} = sub () { @{ $dependend{$name}->{outer} } };
+            *{"$name\::_inner_middlewares"} = sub () { @{ $dependend{$name}->{inner} } };
         } else {
-            push @middlewares, [ $_[$i] => {} ];
+            $dependend{$name}->{outer} = [ $name->_outer_middlewares ];
+            $dependend{$name}->{inner} = [ $name->_inner_middlewares ];
+        }
+
+        my $instance = $name->new($config);
+        @{ $instance->before_handles } = $name->_before_handles;
+        @{ $instance->after_handles }  = $name->_after_handles;
+
+        $self->_instance_of->{$name} = $instance;
+    }
+    # check dependency and sorting
+    my $i = 0;
+    my %sort = map { $_ => $i++ } @{ $self->middlewares };
+    while (my($from, $conf) = each %dependend) {
+        for my $to (@{ $conf->{outer} }) {
+            Carp::croak "'$from' need to '$to'" unless Mouse::is_class_loaded($to);
+            $sort{$to} = $sort{$from} - 1;
+        }
+        for my $to (@{ $conf->{inner} }) {
+            Carp::croak "'$from' need to '$to'" unless Mouse::is_class_loaded($to);
+            $sort{$to} = $sort{$from} + 1;
         }
     }
-
-    return \@middlewares;
-}
-
-# load & create middleware instance
-my %IS_INITIALIZED;
-sub _create_middleware_instance {
-    my ($self, $args) = @_;
-
-    my %instances;
-    for my $stuff (@$args) {
-        my $klass  = $stuff->[0];
-        my $config = $stuff->[1];
-
-        unless ($IS_INITIALIZED{$klass}++) {
-            $self->_init_middleware_class($klass);
-        }
-
-        my $instance = $klass->new(
-            %$config,
-            before_handles => [$klass->_before_handles()],
-            after_handles  => [$klass->_after_handles() ],
-        );
-
-        push @{ $self->_instance_ary_ex }, $instance;
-        push @{ $self->middlewares }, $klass;
-        push @{ $self->_instance_of->{$klass} }, $instance;
-    }
-}
-
-# load one middleware 'class'
-sub _init_middleware_class {
-    my ($self, $klass,) = @_;
-
-    my @before_handles;
-    my @after_handles;
-
-    no warnings 'redefine';
-
-    local *before_handle = sub { push @before_handles, @_ };
-    local *after_handle  = sub { push @after_handles, @_ };
-    local *middleware_method = sub {
-        my($method, $code) = @_;
-        my $method_class = $self->method_class;
-        if ($method =~ /^(.+)\:\:([^\:]+)$/) {
-            ($method_class, $method) = ($1, $2);
-        }
-        return unless $method_class;
-
-        no strict 'refs';
-        *{"$klass\::$method"}        = $code;
-        *{"$method_class\::$method"} = $code;
-    };
-
-    Any::Moose::load_class($klass);
-
-    no strict 'refs';
-    *{"${klass}::_before_handles"}    = sub () { @before_handles    };
-    *{"${klass}::_after_handles"}     = sub () { @after_handles     };
-}
-
-sub is_class_loaded {
-    my $class = shift;
-    return Any::Moose::is_class_loaded($class);
+    @{ $self->middlewares } = sort { $sort{$a} <=> $sort{$b} } keys %sort;
 }
 
 sub instance_of {
     my($self, $name) = @_;
-    my $stuff = $self->_instance_of->{$name};
-    return wantarray ? @{$stuff} : $stuff->[0];
+    $self->_instance_of->{$name};
 }
 
 sub handler {
@@ -204,7 +183,8 @@ sub handler {
         my $res;
         my @run_middlewares;
     LOOP:
-        for my $instance (@{ $self->_instance_ary_ex }) {
+        for my $middleware (@{ $self->middlewares }) {
+            my $instance = $self->_instance_of->{$middleware};
             for my $code (@{ $instance->before_handles }) {
                 my $ret = $code->($self, $instance, $req);
                 if ($ret->isa('HTTP::Engine::Response')) {
@@ -217,8 +197,8 @@ sub handler {
         }
         my $msg;
         unless ($res) {
-            $self->diecatch(0);
             local $@;
+            $self->diecatch(0);
             eval { $res = $handle->($req) };
             $msg = $@ if !$self->diecatch && $@;
         }
@@ -281,11 +261,14 @@ HTTP::Engine::Middleware is official middlewares distribution of HTTP::Engine.
 
 =head1 WISHLIST
 
+Session
+
 Authentication
 
 OpenID
 
-mod_rewrite ( someone write :p )
+Static
+
 
 and more ideas
 
@@ -297,6 +280,8 @@ Daisuke Maki
 
 Tokuhiro Matsuno E<lt>tokuhirom@gmail.comE<gt>
 
+dann
+
 nyarla
 
 marcus
