@@ -23,11 +23,11 @@ has 'HTTP_METHOD_OVERRIDE_HEADER' => (
 
 before_handle {
     my ( $c, $self, $req ) = @_;
-    $self->override_request_method($req);
+    $self->overload_request_method($req);
     $req;
 };
 
-sub override_request_method {
+sub overload_request_method {
     my ( $self, $req ) = @_;
 
     my $method = $req->method;
@@ -39,7 +39,7 @@ sub override_request_method {
         || $req->header( $self->HTTP_METHOD_OVERRIDE_HEADER );
 
     if ( ($overload && grep { $_ eq $overload } @{ $self->HTTP_METHODS } ) != 0 ) {
-        $req->method( uc $overload );
+        $req->method( uc $overload ) if $overload;
     }
     $req;
 }
@@ -50,7 +50,7 @@ __END__
 
 =head1 NAME
 
-HTTP::Engine::Middleware::MethodOverride - simulate HTTP methods by query parameter
+HTTP::Engine::Middleware::MethodOverride - documentation is TODO
 
 =head1 SYNOPSIS
 
@@ -63,12 +63,4 @@ HTTP::Engine::Middleware::MethodOverride - simulate HTTP methods by query parame
         }
     )->run();
 
-=head1 DESCRIPTION
-
-This module simulates the some minor HTTP methods by the query parameter(ex. DELETE, PUT).
-
-=head1 AUTHORS
-
-dann
-
 =cut
